@@ -36,7 +36,7 @@ solid_fill_get_scanline_32 (pixman_image_t *image,
                             uint32_t        mask_bits)
 {
     uint32_t *end = buffer + width;
-    register uint32_t color = ((solid_fill_t *)image)->color;
+    uint32_t color = image->solid.color_32;
 
     while (buffer < end)
 	*(buffer++) = color;
@@ -44,7 +44,24 @@ solid_fill_get_scanline_32 (pixman_image_t *image,
     return;
 }
 
-static source_pict_class_t
+static void
+solid_fill_get_scanline_64 (pixman_image_t *image,
+			    int             x,
+			    int             y,
+			    int             width,
+			    uint32_t *      buffer,
+			    const uint32_t *mask,
+			    uint32_t        mask_bits)
+{
+    uint64_t *b = (uint64_t *)buffer;
+    uint64_t *e = b + width;
+    uint64_t color = image->solid.color_64;
+
+    while (b < e)
+	*(b++) = color;
+}
+
+static source_image_class_t
 solid_fill_classify (pixman_image_t *image,
                      int             x,
                      int             y,
@@ -58,7 +75,7 @@ static void
 solid_fill_property_changed (pixman_image_t *image)
 {
     image->common.get_scanline_32 = solid_fill_get_scanline_32;
-    image->common.get_scanline_64 = _pixman_image_get_scanline_generic_64;
+    image->common.get_scanline_64 = solid_fill_get_scanline_64;
 }
 
 static uint32_t
@@ -71,6 +88,16 @@ color_to_uint32 (const pixman_color_t *color)
         (color->blue >> 8);
 }
 
+static uint64_t
+color_to_uint64 (const pixman_color_t *color)
+{
+    return
+        ((uint64_t)color->alpha << 48) |
+        ((uint64_t)color->red << 32) |
+        ((uint64_t)color->green << 16) |
+        ((uint64_t)color->blue);
+}
+
 PIXMAN_EXPORT pixman_image_t *
 pixman_image_create_solid_fill (pixman_color_t *color)
 {
@@ -80,14 +107,14 @@ pixman_image_create_solid_fill (pixman_color_t *color)
 	return NULL;
 
     img->type = SOLID;
-    img->solid.color = color_to_uint32 (color);
+    img->solid.color = *color;
+    img->solid.color_32 = color_to_uint32 (color);
+    img->solid.color_64 = color_to_uint64 (color);
 
     img->source.class = SOURCE_IMAGE_CLASS_UNKNOWN;
     img->common.classify = solid_fill_classify;
     img->common.property_changed = solid_fill_property_changed;
 
-    solid_fill_property_changed (img);
-
     return img;
 }
 
